# -*- coding: utf-8 -*-
import json
from datetime import datetime
import getopt
import time
import bcesigner
import httplib
import sys
from Crypto.Cipher import AES

def aes_encrypt(sk, text):
    print len(text)
    pad_it = lambda s: s + (16 - len(s) % 16) * chr(16 - len(s) % 16)    
    print pad_it(text)
    key = sk[0:16]    
    mode = AES.MODE_ECB    
    cryptor = AES.new(key, mode, key)    
    cipheradminpass = cryptor.encrypt(pad_it(text)).encode('hex')    
    return cipheradminpass

def genUrl(selfUri, params):
    index = 1
    for param in params:
        if index == 1:
            selfUri += "?"
        else:
            selfUri += "&"
        selfUri = selfUri + param + '=' + str(params[param])
        index += 1
    return selfUri

ak = ''
sk = ''
host = ''

def send_cfs_request(uri, method, params, requestBody):
    global ak
    global sk
    global host

    real_params = {}
    for param in params:
        if params[param] != '':
            real_params[param] = params[param]

    utc_date = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
    signer_obj = bcesigner.ClassBceSigner(ak, sk)

    request = {
            'method': method,
            'uri': uri,
            'params': real_params,
            'headers': {
                'x-bce-date': utc_date,
                'Host': host,
                'content-type': 'application/json'
            }
    }

    req = httplib.HTTPConnection(host)
    request['headers']["Authorization"] = signer_obj.gen_authorization(request)
    myUrl = genUrl(request['uri'], request['params'])
    req.request(request['method'], myUrl, json.dumps(requestBody), request['headers'])
    #print request
    print req

    response = req.getresponse()
    print "httpStatus: " + str(response.status)
    for header in response.getheaders():
        print header
    print "body:"
    responseBody = response.read()

    if responseBody is not None and responseBody.__len__() > 0:
        if response.status == 200:
            print json.dumps(json.loads(responseBody), sort_keys=True, indent=4)
        else:
            print responseBody

# for CFS
def create_cfs(fs_name, vpc_id):
    request_body = {
            "fsName": fs_name,
            "type": "cap",
            "protocol": "nfs",
            "vpcId": vpc_id,
            }
    send_cfs_request("/v1/cfs", "POST", {}, request_body)

def update_cfs(fsid, fs_name):
    if fs_name == '':
            request_body = {
                    }
    else:
            request_body = {
                    "fsName": fs_name,
                    }
    send_cfs_request("/v1/cfs/" + fsid, "PUT", {}, request_body)

def describe_cfs(fsid, user_id, marker, max_keys):
    query_param = {
            "fsId": fsid,
            #"userId": user_id,
            "marker": marker,
            "maxKeys": max_keys
            }
    send_cfs_request("/v1/cfs", "GET", query_param, {})

def drop_cfs(fsid):
    send_cfs_request("/v1/cfs/" + fsid, "DELETE", {}, {})

# for MountTarget
def create_mount_target(fsid, subnet_id):
    request_body = {
            "subnetId": subnet_id,
            }
    send_cfs_request("/v1/cfs/" + fsid, "POST", {}, request_body)

def describe_mount_target(fsid, mount_id, marker, max_keys):
    query_param = {
            "mountId": mount_id,
            "marker": marker,
            "maxKeys": max_keys
            }
    send_cfs_request("/v1/cfs/" + fsid, "GET", query_param, {})

def drop_mount_target(fsid, mount_id):
    send_cfs_request("/v1/cfs/" + fsid + "/" + mount_id, "DELETE", {}, {})


def usage():
    print 'Usage: python cfs.py [options]'
    print 'Example:'
    print '\tCreate cfs:            python cfs.py --command=CreateCFS --region=bj --accessKey=ak --secretKey=sk --fsName=fs_test --vpcId=vpcShortId'
    print '\t    (To create a cfs instance in a valid VPC indicated by vpcId)'
    print '\tList cfs:              python cfs.py --command=ListCFS --region=bj --accessKey=ak--secretKey=sk [--fsId=cfs-8fsHEshLWW]'
    print '\t    (To show the specific cfs instance if fsId is given, otherwise list all the cfs instances of the user)'
    print '\tUpdate cfs:            python cfs.py --command=UpdateCFS --region=bj --accessKey=ak --secretKey=sk --fsId=cfs-8fsHEshLWW [--fsName=new_name]'
    print '\t    (To update the name of the cfs instance if fsName is given, otherwise do nothing)'
    print '\tDrop cfs:              python cfs.py --command=DropCFS --region=bj --accessKey=ak --secretKey=sk --fsId=cfs-8fsHEshLWW'
    print '\t    (To delete the cfs instance indicated by fsId)'
    print '\tCreate mount point:    python cfs.py --command=CreateMountTarget --region=bj --accessKey=ak --secretKey=sk --fsId=cfs-8fsHEshLWW --subnetId=your_subnet_id'
    print '\t    (To create a mount point of the cfs instance in a valid Subnet indicated by subnetId)'
    print '\tList mount point:      python cfs.py --command=ListMountTarget --region=bj --accessKey=ak --secretKey=sk --fsId=cfs-8fsHEshLWW [--mountId=mount_id]'
    print '\t    (To show the specific mount point if mountId is given, otherwise list all the mount points of the cfs instance)'
    print '\tDrop mount point:      python cfs.py --command=DropMountTarget --region=bj --accessKey=ak --secretKey=sk --fsId=cfs-8fsHEshLWW --mountId=mount_id'
    print '\t    (To delete the mount point indicated by mountId)'
    print 'Options:'
    print '\tcommand:      CreateCFS, UpdateCFS, ListCFS, DropCFS, CreateMountTarget, ListMountTarget, DropMountTarget'
    print '\tregion:       bj'
    print '\ttype:         cap(default), ssd'
    print '\tprotocol:     nfs(default), smb' 
    pass

def main(argv):
    VERSION = "2018/03/21 " + "1.0.0.1"
    arguments = {
            "command": "",
            "accessKey": "",
            "secretKey": "",
            "region": "",
            "fsName": "",
            "clientToken": "",
            "type": "",
            "protocol": "",
            "vpcId": "",
            "fsId": "",
            "subnetId": "",
            "marker": "",
            "maxKeys": "",
            "mountId": "",
            "userId": "",
    }

    try:
        long_options = ["help", "version"]
        for argument in arguments.keys():
            long_options.append(argument+"=")

        opts, args = getopt.getopt(argv, "hv", long_options)
    except getopt.GetoptError as err:
        print str(err)
        usage()
        sys.exit(2)

    long_options = []
    for argument in arguments.keys():
        long_options.append("--"+argument)

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-v", "--version"):
            print VERSION
            sys.exit()
        elif opt in long_options:
            arguments[opt[2:]] = arg 

    global ak
    global sk
    global host
    ak = arguments['accessKey']
    sk = arguments['secretKey']
    host = 'cfs.' + arguments['region'] + '.baidubce.com'

    if arguments['command'] == 'CreateCFS':
        create_cfs(arguments['fsName'], arguments['vpcId'])
    elif arguments['command'] == 'UpdateCFS':
        update_cfs(arguments['fsId'], arguments['fsName'])
    elif arguments['command'] == 'ListCFS':
        describe_cfs(arguments['fsId'], arguments['userId'], 
            arguments['marker'], arguments['maxKeys'])
    elif arguments['command'] == 'DropCFS':
        drop_cfs(arguments['fsId'])
    elif arguments['command'] == 'CreateMountTarget':
        create_mount_target(arguments['fsId'], arguments['subnetId'])
    elif arguments['command'] == 'ListMountTarget':
        describe_mount_target(arguments['fsId'], arguments['mountId'], 
            arguments['marker'], arguments['maxKeys'])
    elif arguments['command'] == 'DropMountTarget':
        drop_mount_target(arguments['fsId'], arguments['mountId'])
    else:
        usage()

if __name__ == '__main__':
    main(sys.argv[1:])
